clear

capture log close

log using Section8, replace

use "Chong et al Final Data Set.dta"

d student_id class_level gradesq34 treatment

tab treatment


// generate treatment variable treat=0(control,placebo) treat=1(treatment1,soccer video) treat=2(treatment2,doctor video)
gen treat=treatment

replace treat=0 if treatment==3

//* Table 6
tab treat class_level

//* Table 7
// First, run SAT Regression
quietly car_sat gradesq34 treat, strata(class_level) 

* Collect all info from the regression
mat b=e(b)

mat Vsat=e(V) 

mat VH=e(V_H)

mat Vhc=e(V_hc)

mat VarCov_sat=Vsat*e(N)

mat VarCov_hc=Vhc*e(N)

// Display estimators for variance-covariance matrices
matlist VarCov_sat

matlist VarCov_hc

scalar theta1 = b[1,1]

scalar theta2 = b[1,2]

scalar var_hc1 = Vhc[1,1]

scalar var_hc2 = Vhc[2,2]

scalar cov_hc12 = Vhc[1,2]

scalar cov_sat12 = Vsat[1,2]

scalar var_sat1 = Vsat[1,1]

scalar var_sat2 = Vsat[2,2]

scalar maxi = max(var_sat1/var_hc1,var_sat2/var_hc2)

// Maximimum ratio of variances
di maxi

// Show that covariaces are different
di cov_sat12/cov_hc12

// Run and report SAT regression with new s.e.'s
car_sat gradesq34 treat, strata(class_level)

// Report inference based on hc s.e.'s
// Standard errors for hc s.e.'s
di sqrt(var_hc1) " " sqrt(var_hc2)

// t-statistics for hc s.e.'s
di theta1/sqrt(var_hc1) " " theta2/sqrt(var_hc2) 

// p-values for hc s.e.'s
di 2*ttail(e(df_r),abs(theta1/sqrt(var_hc1)))

di 2*ttail(e(df_r),abs(theta2/sqrt(var_hc2)))

// 95 percent -Confidence intervalss
scalar alpha=0.05

di theta1-invt(e(df_r), 1-alpha/2)*sqrt(var_hc1) " " theta1+invt(e(df_r), 1-alpha/2)*sqrt(var_hc1)

di theta2-invt(e(df_r), 1-alpha/2)*sqrt(var_hc2) " " theta2+invt(e(df_r), 1-alpha/2)*sqrt(var_hc2)

// Run and report SFE regression with new s.e.'s. Note how results are similar to SAT
car_sfe gradesq34 treat, strata(class_level)

log close
