#f Author: Ivan Canay | Northwestern University | December 2015
#-----------------------------------------------------------------------
# X-plain: Computes the simulations for inference under CAR with Expensive Treatment 
# Latest Revision: Dec. 14th, 2015: First version of this file 
#-----------------------------------------------------------------------
# RETURNS:
rm(list=ls(all=TRUE))               # Clear all variables

source("Functions.Expensive.CAR.V2.R")
library(xtable)

#-------------------------------------------------------------------
Expensive.CAR <-function(myseed,n,model=1,gamma=1,sg1=1,num.strata=10,pi=0,theta=0,MC=10000){
#-------------------------------------------------------------------  
 
#IMPUTS: myseed: seed for the simulations 
#		 model: specific DGP for potential outcomes
#        gamma: coefficient in potential outcomes equation
#        sg1: variance of error for potential outcome 1 (this number is sigma-square 1)
#		 num.strata: number of strata 
#        theta: value of theta = mu_1 - mu_0.
#        pi: probability of treatment in each strata
#        MC: Number of monte carlo simulations
#myseed=9999;n=1000;model=1;gamma=1;sg1=1;num.strata=10;pi=0;theta=0;MC=100;


# GENERATE DATA and SET PARAMETERS
# - - - - - - - - - - - - - - - - -
t.pi=round(pi, digits = 2);
dir.create("Tables")
nametext <-paste("Tables/CAR2","M",model,gamma,sg1,t.pi,myseed,"txt",sep=".");
dir.create("Latex")
nameLaTeX<-paste("Latex/CAR2","M",model,gamma,sg1,t.pi,myseed,"tex",sep=".");
dir.create("RData")
Rname 	 <-paste("RData/CAR2","M",model,gamma,sg1,t.pi,myseed,"Rdata",sep=".");
alpha   <-0.05;
cv<-qnorm(1-alpha/2);
set.seed(myseed);

# Set all rejection probabilities to zero
rej.t.srs      <-matrix(0,MC,1); rej.t.wei <-rej.t.srs; rej.t.bcd <-rej.t.srs; rej.t.sbr <-rej.t.srs;
rej.SFEHete.srs <-rej.t.srs; rej.SFEHete.wei<-rej.t.srs; rej.SFEHete.bcd<-rej.t.srs; rej.SFEHete.sbr<-rej.t.srs;
rej.SFEHo.srs <-rej.t.srs; rej.SFEHo.wei<-rej.t.srs; rej.SFEHo.bcd<-rej.t.srs; rej.SFEHo.sbr<-rej.t.srs;
rej.SFEnew.srs <-rej.t.srs; rej.SFEnew.wei<-rej.t.srs; rej.SFEnew.bcd<-rej.t.srs; rej.SFEnew.sbr<-rej.t.srs;
rej.SATHete.srs <-rej.t.srs; rej.SATHete.wei<-rej.t.srs; rej.SATHete.bcd<-rej.t.srs; rej.SATHete.sbr<-rej.t.srs;
rej.SATHo.srs <-rej.t.srs; rej.SATHo.wei<-rej.t.srs; rej.SATHo.bcd<-rej.t.srs; rej.SATHo.sbr<-rej.t.srs;
rej.SATnew.srs <-rej.t.srs; rej.SATnew.wei<-rej.t.srs; rej.SATnew.bcd<-rej.t.srs; rej.SATnew.sbr<-rej.t.srs;
rej.SFEreg.srs <-rej.t.srs; rej.SFEreg.wei<-rej.t.srs; rej.SFEreg.bcd<-rej.t.srs; rej.SFEreg.sbr<-rej.t.srs;
rej.SATreg.srs <-rej.t.srs; rej.SATreg.wei<-rej.t.srs; rej.SATreg.bcd<-rej.t.srs; rej.SATreg.sbr<-rej.t.srs;

sq.H.srs<-rej.t.srs; sq.H.wei<-rej.t.srs; sq.H.bcd<-rej.t.srs; sq.H.sbr<-rej.t.srs;

time1 <- proc.time()
time3 <- proc.time()

if (pi==0){deep.pi=1/2} else {deep.pi=pi}
deep.param<-true.values(model,gamma,sg1,num.strata,deep.pi);

for (j in 1:MC){
	# Generate Baseline covariates and potential outcomes (also Strata)
	base	<- gen.baseline.one(n,model,theta,gamma,sg1);
	I.S 	<-form.strata.one(base,num.strata);

	# Generate treatment assignments using CAR
	data.srs<-dgp.srs(base,I.S,pi);
	data.sbr<-dgp.sbr(base,I.S,pi);
  
  	# (2) Regression with Strata Fixed Effects (SFE in PAPER)
  	SFE.srs <-lm(Y~ A + I.S,data.srs);
	SFE.sbr <-lm(Y~ A + I.S,data.sbr);
	nume.sfe.srs <-sqrt(n)*coef(summary(SFE.srs))[2,1];
	nume.sfe.sbr <-sqrt(n)*coef(summary(SFE.sbr))[2,1];

		# PART (A): HC standard errors
  	rej.SFEHete.srs[j]<-(coeftest(SFE.srs, vcov = vcovHC(SFE.srs,type="HC1"))[2,4]<alpha);
	rej.SFEHete.sbr[j]<-(coeftest(SFE.sbr, vcov = vcovHC(SFE.sbr,type="HC1"))[2,4]<alpha);
   
		# PART (B): homoskedastic standard errors
	rej.SFEHo.srs[j]<-(coef(summary(SFE.srs))[2,4]<alpha);
	rej.SFEHo.sbr[j]<-(coef(summary(SFE.sbr))[2,4]<alpha);  

		# PART (C): Simply record SE for new estimator later on
	homo.sfe.srs <-n*coef(summary(SFE.srs))[2,2]^2;
	homo.sfe.sbr <-n*coef(summary(SFE.sbr))[2,2]^2;
	hete.sfe.srs <-n*coeftest(SFE.srs, vcov = vcovHC(SFE.srs,type="HC1"))[2,2]^2;
	hete.sfe.sbr <-n*coeftest(SFE.sbr, vcov = vcovHC(SFE.sbr,type="HC1"))[2,2]^2;

	# (3) Fully Saturated Regression (SAT in paper)
	SAT.srs <- lm(Y~.+0,data.srs[,-c(2,3)]);
	SAT.sbr <- lm(Y~.+0,data.sbr[,-c(2,3)]);
	sum.SAT.srs <-summary(SAT.srs);
   	sum.SAT.sbr <-summary(SAT.sbr);

   	weights<-c(rep(0,num.strata),colMeans(I.S));
   	R.srs  <- matrix(weights[!is.na(coef(SAT.srs))]);
   	R.sbr  <- matrix(weights[!is.na(coef(SAT.sbr))]);
   	if (sum(R.srs)==0){caca=caca+1;}

   	nume.sat.srs<- sqrt(n)*sum(R.srs*coef(sum.SAT.srs)[,1]);
	nume.sat.sbr<- sqrt(n)*sum(R.sbr*coef(sum.SAT.sbr)[,1]);

	homo.sat.srs<- n*t(R.srs)%*%vcovHC(SAT.srs,type="const")%*%R.srs;
	homo.sat.sbr<- n*t(R.sbr)%*%vcovHC(SAT.sbr,type="const")%*%R.sbr;

	hete.sat.srs<- n*t(R.srs)%*%vcovHC(SAT.srs,type="HC1")%*%R.srs;
	hete.sat.sbr<- n*t(R.sbr)%*%vcovHC(SAT.sbr,type="HC1")%*%R.sbr;

		# PART (A): HC standard errors
  	rej.SATHete.srs[j]<-(abs(nume.sat.srs)/sqrt(hete.sat.srs)>cv);
	rej.SATHete.sbr[j]<-(abs(nume.sat.sbr)/sqrt(hete.sat.sbr)>cv);

		# PART (B): homoskedastic standard errors
  	rej.SATHo.srs[j]<-(abs(nume.sat.srs)/sqrt(homo.sat.srs)>cv);
	rej.SATHo.sbr[j]<-(abs(nume.sat.sbr)/sqrt(homo.sat.sbr)>cv);

	# (4) Rejections with NEW Standard Errors (This paper)
	sq.H.srs[j] <- max(homo.sfe.srs-homo.sat.srs,0);
	sq.H.sbr[j] <- max(homo.sfe.sbr-homo.sat.sbr,0);
	
	if (pi==0){mean.pi=1/2};
	
		# NEW VARIANCE ESTIMATOR FOR SFE
	#wei.factor  <-1/(1+4*pi);
	new.sfe.srs <- hete.sat.srs + sq.H.srs[j] + (1-2*mean.pi)^2*sq.H.srs[j]/(mean.pi*(1-mean.pi));
	new.sfe.sbr <- hete.sat.sbr + sq.H.sbr[j];
	new.sat.srs <- hete.sat.srs + sq.H.srs[j];
	new.sat.sbr <- hete.sat.sbr + sq.H.sbr[j];

		# PART (A): SFE Rejections
  	rej.SFEnew.srs[j]<-(abs(nume.sfe.srs)/sqrt(new.sfe.srs)>cv);
	rej.SFEnew.sbr[j]<-(abs(nume.sfe.sbr)/sqrt(new.sfe.sbr)>cv);

		# PART (B): SAT Rejections 
  	rej.SATnew.srs[j]<-(abs(nume.sat.srs)/sqrt(new.sat.srs)>cv);
	rej.SATnew.sbr[j]<-(abs(nume.sat.sbr)/sqrt(new.sat.sbr)>cv);	
 
	# Report Iterations on Screen for monitoring
	if (floor(10*j/MC)==10*j/MC){
    cat("% completed = ", j/MC*100, "\n")
	  time4 <- proc.time();
    cat("Time Last round (min) = ", (time4[3]-time3[3])/60, "\n")
    cat(mean(rej.SFEnew.sbr),"\n")
    time3 <-proc.time()
	} 
  
}

time2 <- proc.time();
cat(sprintf("Elapsed Time (min): %1.3f",(time2[3]-time1[3])/60), "\n");

## Display Information
sq.H<-round(mean(sq.H.srs),digits = 2);
basics_a <- c(model,n,theta,gamma,sg1,num.strata,alpha,t.pi,sq.H,MC);
names(basics_a)<- c("model", "n  ", "theta", "gamma", "sg1", "# strata", "alpha", "pi", "Ssq H", " MC iterations");

## Null rejection probabilities
SRS = c(mean(rej.SATHo.srs)*100,mean(rej.SATHete.srs)*100,mean(rej.SATnew.srs)*100,mean(rej.SFEHo.srs)*100,mean(rej.SFEHete.srs)*100,mean(rej.SFEnew.srs)*100);
SBR = c(mean(rej.SATHo.sbr)*100,mean(rej.SATHete.sbr)*100,mean(rej.SATnew.sbr)*100,mean(rej.SFEHo.sbr)*100,mean(rej.SFEHete.sbr)*100,mean(rej.SFEnew.sbr)*100);
names(SRS)  <-c("        SAT homo |", "   SAT Hete  |", "   SAT NEW  |", "    SFE Homo     |","    SFE Hete      |","  SFE New  ");
results <-rbind(SRS,SBR);

# Save LaTeX table
X.results<-cbind(c("SRS","SBR"),results)
row.names(X.results)<-c(model," ");
X.results<-xtable(X.results)
print(X.results, type="latex", file=nameLaTeX);

print(results)

# # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# Saving information to a text file
out <- file(nametext,"wt");   # Choose name of the file here
cat("---------------------------------------------------------------------------------------------------------------------", "\n",file=out);
cat(date(), "\n",file=out)
cat("Seed used in this round = ", myseed , "\n",file=out)
cat("\n",file=out)
cat("----------------------------------------------- Parameters ----------------------------------------------------------", "\n",file=out);
cat(names(basics_a),sep="   ","\n",file=out);
cat(formatC(basics_a,digits=2,format="f"),sep="   ", "\n",file=out);
cat("\n",file=out)
  cat(c('Var of Y(1) =',round(deep.param$VY1,digits = 2)), "\n", file=out)
  cat(c('Var of Y(0) =',round(deep.param$VY0,digits = 2)), "\n", file=out)
  cat(c('Var of til Y(1) =',round(deep.param$VTY1,digits = 2)), "\n", file=out)
  cat(c('Var of til Y(0) =',round(deep.param$VTY0,digits = 2)), "\n", file=out)
  cat(c('Var of Y =',round(deep.param$vsgY,digits = 2) ), "\n", file=out)
  cat(c('Var of tilde Y =',round(deep.param$vsgTY,digits = 2) ), "\n", file=out)
  cat(c('Var of H =',round(deep.param$vsgH,digits = 2)), "\n", file=out)
  cat(c('Var of A =',round(deep.param$vsgA,digits = 2)), "\n", file=out)
cat("\n",file=out)
cat("---------------------------------------- NULL REJECTION PROBABILITIES ----------------------------------------------", "\n",file=out);
cat(names(SRS),sep=" ","\n",file=out);
write.table(format(results, drop0trailing=FALSE),file=out,quote=F,sep='\t\t',row.names=T,col.names=F)
cat("\n",file=out)
cat("---------------------------------------------------------------------------------------------------------------------", "\n",file=out);
time2 <- proc.time();
cat(sprintf("Elapsed Time (min): %1.3f",(time2[3]-time1[3])/60), "\n",file=out);
cat("---------------------------------------------------------------------------------------------------------------------", "\n",file=out);
cat(" SRS: simple random sampling. WEI: wei's design. BCD: bias coin design. SBR: Stratified Block Randomization ", "\n",file=out);
cat(" Model 1: linear model with normal epsi and beta W. ", "\n",file=out);
cat(" Model 2: Same 1 but Heterogenous ", "\n",file=out);
cat(" Model 3: Same 2 but Heteroskedastic ", "\n",file=out);
cat(" Model 4: Uniform W, non-normal errors (t_3), Heterogeneous, Heteroskedastic ", "\n",file=out);
cat("---------------------------------------------------------------------------------------------------------------------", "\n",file=out);
close(out);

save(list = ls(all = TRUE), file = Rname)

}